import json
import logging
import os

from mir import scm
from mir.tools import mir_storage


def mir_check_repo_dvc_dirty(mir_root: str = ".") -> bool:
    names = [name for name in mir_storage.get_all_mir_paths() if os.path.isfile(os.path.join(mir_root, name))]
    if names:
        dvc_cmd_args = ["--show-json", "--targets"]
        dvc_cmd_args.extend(names)
        dvc_scm = scm.Scm(mir_root, scm_executable="dvc")
        dvc_result = dvc_scm.diff(dvc_cmd_args)
        json_object = json.loads(dvc_result)
        dvc_dirty = bool(
            json_object.get("added", None) or json_object.get("deleted", None) or json_object.get("modified", None)
            or json_object.get("renamed", None) or json_object.get("not in cache", None))
        return dvc_dirty
    else:
        # if no mir files in this mir repo, it's clean
        return False


def mir_check_repo_git_dirty(mir_root: str = ".") -> bool:
    git_scm = scm.Scm(mir_root, scm_executable="git")
    git_result = git_scm.status("-s")  # if clean, returns nothing
    if (git_result or len(git_result) > 0):
        logging.debug("git {} dirty, unmatched: {}".format(mir_root, git_result))
        return True
    return False  # clean


def mir_check_repo_dirty(mir_root: str = '.') -> bool:
    return mir_check_repo_dvc_dirty(mir_root) or mir_check_repo_git_dirty(mir_root)


def mir_check_branch_exists(mir_root: str, branch: str) -> bool:
    try:
        git_scm = scm.Scm(mir_root, scm_executable="git")
        git_scm.rev_parse(branch)
        return True
    except Exception:
        # git rev-parse will return non-zero code when can not find branch
        # and cmd.py packs non-zero return code as an error
        return False



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mir_entities.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ymir_proto.gen import mir_common_pb2 as mir__common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='mir_entities.proto',
  package='ymir.entities',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12mir_entities.proto\x12\rymir.entities\x1a\x10mir_common.proto\"\xa5\x01\n\x0cMirMetadatas\x12?\n\nattributes\x18\x01 \x03(\x0b\x32+.ymir.entities.MirMetadatas.AttributesEntry\x1aT\n\x0f\x41ttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x30\n\x05value\x18\x02 \x01(\x0b\x32!.ymir.entities.MetadataAttributes:\x02\x38\x01\"\xe2\x01\n\x12MetadataAttributes\x12\x14\n\x0c\x64\x61taset_name\x18\x01 \x01(\t\x12+\n\ttimestamp\x18\x02 \x01(\x0b\x32\x18.ymir.entities.Timestamp\x12&\n\x08tvt_type\x18\x03 \x01(\x0e\x32\x14.ymir.common.TvtType\x12*\n\nasset_type\x18\x04 \x01(\x0e\x32\x16.ymir.common.AssetType\x12\r\n\x05width\x18\x05 \x01(\x05\x12\x0e\n\x06height\x18\x06 \x01(\x05\x12\x16\n\x0eimage_channels\x18\x07 \x01(\x05\",\n\tTimestamp\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x10\n\x08\x64uration\x18\x02 \x01(\x02\"\xd2\x01\n\x0eMirAnnotations\x12L\n\x10task_annotations\x18\x01 \x03(\x0b\x32\x32.ymir.entities.MirAnnotations.TaskAnnotationsEntry\x12\x14\n\x0chead_task_id\x18\x02 \x01(\t\x1a\\\n\x14TaskAnnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.ymir.entities.SingleTaskAnnotations:\x02\x38\x01\"\xce\x01\n\x15SingleTaskAnnotations\x12U\n\x11image_annotations\x18\x01 \x03(\x0b\x32:.ymir.entities.SingleTaskAnnotations.ImageAnnotationsEntry\x1a^\n\x15ImageAnnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.ymir.entities.SingleImageAnnotations:\x02\x38\x01\"H\n\x16SingleImageAnnotations\x12.\n\x0b\x61nnotations\x18\x02 \x03(\x0b\x32\x19.ymir.entities.Annotation\"^\n\nAnnotation\x12\r\n\x05index\x18\x01 \x01(\x05\x12 \n\x03\x62ox\x18\x02 \x01(\x0b\x32\x13.ymir.entities.Rect\x12\x10\n\x08\x63lass_id\x18\x03 \x01(\x05\x12\r\n\x05score\x18\x04 \x01(\x01\"2\n\x04Rect\x12\t\n\x01x\x18\x01 \x01(\x05\x12\t\n\x01y\x18\x02 \x01(\x05\x12\t\n\x01w\x18\x03 \x01(\x05\x12\t\n\x01h\x18\x04 \x01(\x05\"\xaa\x05\n\x0bMirKeywords\x12:\n\x08keywords\x18\x01 \x03(\x0b\x32(.ymir.entities.MirKeywords.KeywordsEntry\x12R\n\x15predifined_keyids_cnt\x18\x02 \x03(\x0b\x32\x33.ymir.entities.MirKeywords.PredifinedKeyidsCntEntry\x12\x1f\n\x17predifined_keyids_total\x18\x03 \x01(\x05\x12V\n\x17\x63ustomized_keywords_cnt\x18\x04 \x03(\x0b\x32\x35.ymir.entities.MirKeywords.CustomizedKeywordsCntEntry\x12!\n\x19\x63ustomized_keywords_total\x18\x05 \x01(\x05\x12V\n\x17index_predifined_keyids\x18\x06 \x03(\x0b\x32\x35.ymir.entities.MirKeywords.IndexPredifinedKeyidsEntry\x1aH\n\rKeywordsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.ymir.entities.Keywords:\x02\x38\x01\x1a:\n\x18PredifinedKeyidsCntEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a<\n\x1a\x43ustomizedKeywordsCntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1aS\n\x1aIndexPredifinedKeyidsEntry\x12\x0b\n\x03key\x18\x01 \x01(\x05\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.ymir.entities.Assets:\x02\x38\x01\"\x1b\n\x06\x41ssets\x12\x11\n\tasset_ids\x18\x01 \x03(\t\"B\n\x08Keywords\x12\x19\n\x11predifined_keyids\x18\x01 \x03(\x05\x12\x1b\n\x13\x63ustomized_keywords\x18\x02 \x03(\t\"\x96\x01\n\x08MirTasks\x12\x31\n\x05tasks\x18\x01 \x03(\x0b\x32\".ymir.entities.MirTasks.TasksEntry\x12\x14\n\x0chead_task_id\x18\x02 \x01(\t\x1a\x41\n\nTasksEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.ymir.entities.Task:\x02\x38\x01\"\xc4\x01\n\x04Task\x12#\n\x04type\x18\x01 \x01(\x0e\x32\x15.ymir.common.TaskType\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07task_id\x18\x03 \x01(\t\x12\x14\n\x0c\x62\x61se_task_id\x18\x04 \x01(\t\x12\x11\n\ttimestamp\x18\x05 \x01(\x03\x12\'\n\x05model\x18\x06 \x01(\x0b\x32\x18.ymir.entities.ModelMeta\x12\x0c\n\x04\x61rgs\x18\x64 \x01(\t\x12\x18\n\x10\x61ncestor_task_id\x18\x65 \x01(\t\"P\n\tModelMeta\x12\x12\n\nmodel_hash\x18\x01 \x01(\t\x12\x1e\n\x16mean_average_precision\x18\x02 \x01(\x02\x12\x0f\n\x07\x63ontext\x18\x03 \x01(\tb\x06proto3'
  ,
  dependencies=[mir__common__pb2.DESCRIPTOR,])




_MIRMETADATAS_ATTRIBUTESENTRY = _descriptor.Descriptor(
  name='AttributesEntry',
  full_name='ymir.entities.MirMetadatas.AttributesEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirMetadatas.AttributesEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirMetadatas.AttributesEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=221,
)

_MIRMETADATAS = _descriptor.Descriptor(
  name='MirMetadatas',
  full_name='ymir.entities.MirMetadatas',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributes', full_name='ymir.entities.MirMetadatas.attributes', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MIRMETADATAS_ATTRIBUTESENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=221,
)


_METADATAATTRIBUTES = _descriptor.Descriptor(
  name='MetadataAttributes',
  full_name='ymir.entities.MetadataAttributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset_name', full_name='ymir.entities.MetadataAttributes.dataset_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='ymir.entities.MetadataAttributes.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tvt_type', full_name='ymir.entities.MetadataAttributes.tvt_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='asset_type', full_name='ymir.entities.MetadataAttributes.asset_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='width', full_name='ymir.entities.MetadataAttributes.width', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='height', full_name='ymir.entities.MetadataAttributes.height', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='image_channels', full_name='ymir.entities.MetadataAttributes.image_channels', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=224,
  serialized_end=450,
)


_TIMESTAMP = _descriptor.Descriptor(
  name='Timestamp',
  full_name='ymir.entities.Timestamp',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='start', full_name='ymir.entities.Timestamp.start', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='duration', full_name='ymir.entities.Timestamp.duration', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=452,
  serialized_end=496,
)


_MIRANNOTATIONS_TASKANNOTATIONSENTRY = _descriptor.Descriptor(
  name='TaskAnnotationsEntry',
  full_name='ymir.entities.MirAnnotations.TaskAnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirAnnotations.TaskAnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirAnnotations.TaskAnnotationsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=617,
  serialized_end=709,
)

_MIRANNOTATIONS = _descriptor.Descriptor(
  name='MirAnnotations',
  full_name='ymir.entities.MirAnnotations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='task_annotations', full_name='ymir.entities.MirAnnotations.task_annotations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='head_task_id', full_name='ymir.entities.MirAnnotations.head_task_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MIRANNOTATIONS_TASKANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=499,
  serialized_end=709,
)


_SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY = _descriptor.Descriptor(
  name='ImageAnnotationsEntry',
  full_name='ymir.entities.SingleTaskAnnotations.ImageAnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.SingleTaskAnnotations.ImageAnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.SingleTaskAnnotations.ImageAnnotationsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=824,
  serialized_end=918,
)

_SINGLETASKANNOTATIONS = _descriptor.Descriptor(
  name='SingleTaskAnnotations',
  full_name='ymir.entities.SingleTaskAnnotations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='image_annotations', full_name='ymir.entities.SingleTaskAnnotations.image_annotations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=712,
  serialized_end=918,
)


_SINGLEIMAGEANNOTATIONS = _descriptor.Descriptor(
  name='SingleImageAnnotations',
  full_name='ymir.entities.SingleImageAnnotations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='annotations', full_name='ymir.entities.SingleImageAnnotations.annotations', index=0,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=920,
  serialized_end=992,
)


_ANNOTATION = _descriptor.Descriptor(
  name='Annotation',
  full_name='ymir.entities.Annotation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='index', full_name='ymir.entities.Annotation.index', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='box', full_name='ymir.entities.Annotation.box', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='class_id', full_name='ymir.entities.Annotation.class_id', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='score', full_name='ymir.entities.Annotation.score', index=3,
      number=4, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=994,
  serialized_end=1088,
)


_RECT = _descriptor.Descriptor(
  name='Rect',
  full_name='ymir.entities.Rect',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='ymir.entities.Rect.x', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='ymir.entities.Rect.y', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='w', full_name='ymir.entities.Rect.w', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='h', full_name='ymir.entities.Rect.h', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1090,
  serialized_end=1140,
)


_MIRKEYWORDS_KEYWORDSENTRY = _descriptor.Descriptor(
  name='KeywordsEntry',
  full_name='ymir.entities.MirKeywords.KeywordsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirKeywords.KeywordsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirKeywords.KeywordsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1546,
  serialized_end=1618,
)

_MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY = _descriptor.Descriptor(
  name='PredifinedKeyidsCntEntry',
  full_name='ymir.entities.MirKeywords.PredifinedKeyidsCntEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirKeywords.PredifinedKeyidsCntEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirKeywords.PredifinedKeyidsCntEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1620,
  serialized_end=1678,
)

_MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY = _descriptor.Descriptor(
  name='CustomizedKeywordsCntEntry',
  full_name='ymir.entities.MirKeywords.CustomizedKeywordsCntEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirKeywords.CustomizedKeywordsCntEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirKeywords.CustomizedKeywordsCntEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1680,
  serialized_end=1740,
)

_MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY = _descriptor.Descriptor(
  name='IndexPredifinedKeyidsEntry',
  full_name='ymir.entities.MirKeywords.IndexPredifinedKeyidsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirKeywords.IndexPredifinedKeyidsEntry.key', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirKeywords.IndexPredifinedKeyidsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1742,
  serialized_end=1825,
)

_MIRKEYWORDS = _descriptor.Descriptor(
  name='MirKeywords',
  full_name='ymir.entities.MirKeywords',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='keywords', full_name='ymir.entities.MirKeywords.keywords', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predifined_keyids_cnt', full_name='ymir.entities.MirKeywords.predifined_keyids_cnt', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predifined_keyids_total', full_name='ymir.entities.MirKeywords.predifined_keyids_total', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='customized_keywords_cnt', full_name='ymir.entities.MirKeywords.customized_keywords_cnt', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='customized_keywords_total', full_name='ymir.entities.MirKeywords.customized_keywords_total', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='index_predifined_keyids', full_name='ymir.entities.MirKeywords.index_predifined_keyids', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MIRKEYWORDS_KEYWORDSENTRY, _MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY, _MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY, _MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1143,
  serialized_end=1825,
)


_ASSETS = _descriptor.Descriptor(
  name='Assets',
  full_name='ymir.entities.Assets',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='asset_ids', full_name='ymir.entities.Assets.asset_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1827,
  serialized_end=1854,
)


_KEYWORDS = _descriptor.Descriptor(
  name='Keywords',
  full_name='ymir.entities.Keywords',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predifined_keyids', full_name='ymir.entities.Keywords.predifined_keyids', index=0,
      number=1, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='customized_keywords', full_name='ymir.entities.Keywords.customized_keywords', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1856,
  serialized_end=1922,
)


_MIRTASKS_TASKSENTRY = _descriptor.Descriptor(
  name='TasksEntry',
  full_name='ymir.entities.MirTasks.TasksEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='ymir.entities.MirTasks.TasksEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='ymir.entities.MirTasks.TasksEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=b'8\001',
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2010,
  serialized_end=2075,
)

_MIRTASKS = _descriptor.Descriptor(
  name='MirTasks',
  full_name='ymir.entities.MirTasks',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='tasks', full_name='ymir.entities.MirTasks.tasks', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='head_task_id', full_name='ymir.entities.MirTasks.head_task_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_MIRTASKS_TASKSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1925,
  serialized_end=2075,
)


_TASK = _descriptor.Descriptor(
  name='Task',
  full_name='ymir.entities.Task',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='ymir.entities.Task.type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='ymir.entities.Task.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='task_id', full_name='ymir.entities.Task.task_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='base_task_id', full_name='ymir.entities.Task.base_task_id', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='ymir.entities.Task.timestamp', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='model', full_name='ymir.entities.Task.model', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='args', full_name='ymir.entities.Task.args', index=6,
      number=100, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ancestor_task_id', full_name='ymir.entities.Task.ancestor_task_id', index=7,
      number=101, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2078,
  serialized_end=2274,
)


_MODELMETA = _descriptor.Descriptor(
  name='ModelMeta',
  full_name='ymir.entities.ModelMeta',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model_hash', full_name='ymir.entities.ModelMeta.model_hash', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mean_average_precision', full_name='ymir.entities.ModelMeta.mean_average_precision', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='context', full_name='ymir.entities.ModelMeta.context', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2276,
  serialized_end=2356,
)

_MIRMETADATAS_ATTRIBUTESENTRY.fields_by_name['value'].message_type = _METADATAATTRIBUTES
_MIRMETADATAS_ATTRIBUTESENTRY.containing_type = _MIRMETADATAS
_MIRMETADATAS.fields_by_name['attributes'].message_type = _MIRMETADATAS_ATTRIBUTESENTRY
_METADATAATTRIBUTES.fields_by_name['timestamp'].message_type = _TIMESTAMP
_METADATAATTRIBUTES.fields_by_name['tvt_type'].enum_type = mir__common__pb2._TVTTYPE
_METADATAATTRIBUTES.fields_by_name['asset_type'].enum_type = mir__common__pb2._ASSETTYPE
_MIRANNOTATIONS_TASKANNOTATIONSENTRY.fields_by_name['value'].message_type = _SINGLETASKANNOTATIONS
_MIRANNOTATIONS_TASKANNOTATIONSENTRY.containing_type = _MIRANNOTATIONS
_MIRANNOTATIONS.fields_by_name['task_annotations'].message_type = _MIRANNOTATIONS_TASKANNOTATIONSENTRY
_SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY.fields_by_name['value'].message_type = _SINGLEIMAGEANNOTATIONS
_SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY.containing_type = _SINGLETASKANNOTATIONS
_SINGLETASKANNOTATIONS.fields_by_name['image_annotations'].message_type = _SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY
_SINGLEIMAGEANNOTATIONS.fields_by_name['annotations'].message_type = _ANNOTATION
_ANNOTATION.fields_by_name['box'].message_type = _RECT
_MIRKEYWORDS_KEYWORDSENTRY.fields_by_name['value'].message_type = _KEYWORDS
_MIRKEYWORDS_KEYWORDSENTRY.containing_type = _MIRKEYWORDS
_MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY.containing_type = _MIRKEYWORDS
_MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY.containing_type = _MIRKEYWORDS
_MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY.fields_by_name['value'].message_type = _ASSETS
_MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY.containing_type = _MIRKEYWORDS
_MIRKEYWORDS.fields_by_name['keywords'].message_type = _MIRKEYWORDS_KEYWORDSENTRY
_MIRKEYWORDS.fields_by_name['predifined_keyids_cnt'].message_type = _MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY
_MIRKEYWORDS.fields_by_name['customized_keywords_cnt'].message_type = _MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY
_MIRKEYWORDS.fields_by_name['index_predifined_keyids'].message_type = _MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY
_MIRTASKS_TASKSENTRY.fields_by_name['value'].message_type = _TASK
_MIRTASKS_TASKSENTRY.containing_type = _MIRTASKS
_MIRTASKS.fields_by_name['tasks'].message_type = _MIRTASKS_TASKSENTRY
_TASK.fields_by_name['type'].enum_type = mir__common__pb2._TASKTYPE
_TASK.fields_by_name['model'].message_type = _MODELMETA
DESCRIPTOR.message_types_by_name['MirMetadatas'] = _MIRMETADATAS
DESCRIPTOR.message_types_by_name['MetadataAttributes'] = _METADATAATTRIBUTES
DESCRIPTOR.message_types_by_name['Timestamp'] = _TIMESTAMP
DESCRIPTOR.message_types_by_name['MirAnnotations'] = _MIRANNOTATIONS
DESCRIPTOR.message_types_by_name['SingleTaskAnnotations'] = _SINGLETASKANNOTATIONS
DESCRIPTOR.message_types_by_name['SingleImageAnnotations'] = _SINGLEIMAGEANNOTATIONS
DESCRIPTOR.message_types_by_name['Annotation'] = _ANNOTATION
DESCRIPTOR.message_types_by_name['Rect'] = _RECT
DESCRIPTOR.message_types_by_name['MirKeywords'] = _MIRKEYWORDS
DESCRIPTOR.message_types_by_name['Assets'] = _ASSETS
DESCRIPTOR.message_types_by_name['Keywords'] = _KEYWORDS
DESCRIPTOR.message_types_by_name['MirTasks'] = _MIRTASKS
DESCRIPTOR.message_types_by_name['Task'] = _TASK
DESCRIPTOR.message_types_by_name['ModelMeta'] = _MODELMETA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MirMetadatas = _reflection.GeneratedProtocolMessageType('MirMetadatas', (_message.Message,), {

  'AttributesEntry' : _reflection.GeneratedProtocolMessageType('AttributesEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRMETADATAS_ATTRIBUTESENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirMetadatas.AttributesEntry)
    })
  ,
  'DESCRIPTOR' : _MIRMETADATAS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.MirMetadatas)
  })
_sym_db.RegisterMessage(MirMetadatas)
_sym_db.RegisterMessage(MirMetadatas.AttributesEntry)

MetadataAttributes = _reflection.GeneratedProtocolMessageType('MetadataAttributes', (_message.Message,), {
  'DESCRIPTOR' : _METADATAATTRIBUTES,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.MetadataAttributes)
  })
_sym_db.RegisterMessage(MetadataAttributes)

Timestamp = _reflection.GeneratedProtocolMessageType('Timestamp', (_message.Message,), {
  'DESCRIPTOR' : _TIMESTAMP,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Timestamp)
  })
_sym_db.RegisterMessage(Timestamp)

MirAnnotations = _reflection.GeneratedProtocolMessageType('MirAnnotations', (_message.Message,), {

  'TaskAnnotationsEntry' : _reflection.GeneratedProtocolMessageType('TaskAnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRANNOTATIONS_TASKANNOTATIONSENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirAnnotations.TaskAnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _MIRANNOTATIONS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.MirAnnotations)
  })
_sym_db.RegisterMessage(MirAnnotations)
_sym_db.RegisterMessage(MirAnnotations.TaskAnnotationsEntry)

SingleTaskAnnotations = _reflection.GeneratedProtocolMessageType('SingleTaskAnnotations', (_message.Message,), {

  'ImageAnnotationsEntry' : _reflection.GeneratedProtocolMessageType('ImageAnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.SingleTaskAnnotations.ImageAnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _SINGLETASKANNOTATIONS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.SingleTaskAnnotations)
  })
_sym_db.RegisterMessage(SingleTaskAnnotations)
_sym_db.RegisterMessage(SingleTaskAnnotations.ImageAnnotationsEntry)

SingleImageAnnotations = _reflection.GeneratedProtocolMessageType('SingleImageAnnotations', (_message.Message,), {
  'DESCRIPTOR' : _SINGLEIMAGEANNOTATIONS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.SingleImageAnnotations)
  })
_sym_db.RegisterMessage(SingleImageAnnotations)

Annotation = _reflection.GeneratedProtocolMessageType('Annotation', (_message.Message,), {
  'DESCRIPTOR' : _ANNOTATION,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Annotation)
  })
_sym_db.RegisterMessage(Annotation)

Rect = _reflection.GeneratedProtocolMessageType('Rect', (_message.Message,), {
  'DESCRIPTOR' : _RECT,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Rect)
  })
_sym_db.RegisterMessage(Rect)

MirKeywords = _reflection.GeneratedProtocolMessageType('MirKeywords', (_message.Message,), {

  'KeywordsEntry' : _reflection.GeneratedProtocolMessageType('KeywordsEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRKEYWORDS_KEYWORDSENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirKeywords.KeywordsEntry)
    })
  ,

  'PredifinedKeyidsCntEntry' : _reflection.GeneratedProtocolMessageType('PredifinedKeyidsCntEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirKeywords.PredifinedKeyidsCntEntry)
    })
  ,

  'CustomizedKeywordsCntEntry' : _reflection.GeneratedProtocolMessageType('CustomizedKeywordsCntEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirKeywords.CustomizedKeywordsCntEntry)
    })
  ,

  'IndexPredifinedKeyidsEntry' : _reflection.GeneratedProtocolMessageType('IndexPredifinedKeyidsEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirKeywords.IndexPredifinedKeyidsEntry)
    })
  ,
  'DESCRIPTOR' : _MIRKEYWORDS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.MirKeywords)
  })
_sym_db.RegisterMessage(MirKeywords)
_sym_db.RegisterMessage(MirKeywords.KeywordsEntry)
_sym_db.RegisterMessage(MirKeywords.PredifinedKeyidsCntEntry)
_sym_db.RegisterMessage(MirKeywords.CustomizedKeywordsCntEntry)
_sym_db.RegisterMessage(MirKeywords.IndexPredifinedKeyidsEntry)

Assets = _reflection.GeneratedProtocolMessageType('Assets', (_message.Message,), {
  'DESCRIPTOR' : _ASSETS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Assets)
  })
_sym_db.RegisterMessage(Assets)

Keywords = _reflection.GeneratedProtocolMessageType('Keywords', (_message.Message,), {
  'DESCRIPTOR' : _KEYWORDS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Keywords)
  })
_sym_db.RegisterMessage(Keywords)

MirTasks = _reflection.GeneratedProtocolMessageType('MirTasks', (_message.Message,), {

  'TasksEntry' : _reflection.GeneratedProtocolMessageType('TasksEntry', (_message.Message,), {
    'DESCRIPTOR' : _MIRTASKS_TASKSENTRY,
    '__module__' : 'mir_entities_pb2'
    # @@protoc_insertion_point(class_scope:ymir.entities.MirTasks.TasksEntry)
    })
  ,
  'DESCRIPTOR' : _MIRTASKS,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.MirTasks)
  })
_sym_db.RegisterMessage(MirTasks)
_sym_db.RegisterMessage(MirTasks.TasksEntry)

Task = _reflection.GeneratedProtocolMessageType('Task', (_message.Message,), {
  'DESCRIPTOR' : _TASK,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.Task)
  })
_sym_db.RegisterMessage(Task)

ModelMeta = _reflection.GeneratedProtocolMessageType('ModelMeta', (_message.Message,), {
  'DESCRIPTOR' : _MODELMETA,
  '__module__' : 'mir_entities_pb2'
  # @@protoc_insertion_point(class_scope:ymir.entities.ModelMeta)
  })
_sym_db.RegisterMessage(ModelMeta)


_MIRMETADATAS_ATTRIBUTESENTRY._options = None
_MIRANNOTATIONS_TASKANNOTATIONSENTRY._options = None
_SINGLETASKANNOTATIONS_IMAGEANNOTATIONSENTRY._options = None
_MIRKEYWORDS_KEYWORDSENTRY._options = None
_MIRKEYWORDS_PREDIFINEDKEYIDSCNTENTRY._options = None
_MIRKEYWORDS_CUSTOMIZEDKEYWORDSCNTENTRY._options = None
_MIRKEYWORDS_INDEXPREDIFINEDKEYIDSENTRY._options = None
_MIRTASKS_TASKSENTRY._options = None
# @@protoc_insertion_point(module_scope)



import datetime
import logging
import os
import pathlib
import shutil
import subprocess
import tarfile
from typing import List, Optional, Set, Tuple

import yaml

from mir.commands import base
from mir.tools import checker, data_exporter, hash_utils, mir_storage_ops, revs_parser
from mir.tools.code import MirCode
from ymir_proto.gen import mir_common_pb2 as mir_common
from ymir_proto.gen import mir_entities_pb2 as mirpb
from ymir_proto.util import ids


def _process_model_storage(export_root: str, model_upload_location: str) -> str:
    models = _find_models(os.path.join(export_root, "models"))
    model_paths = models[0]
    model_mAP = models[1]
    if not model_paths:
        raise ValueError("can not find models")
    pack_model_path = os.path.join(export_root, "models.tar.gz")
    _pack_models(model_paths, pack_model_path)
    model_sha1 = hash_utils.sha1sum_for_file(pack_model_path)

    dest_path = os.path.join(model_upload_location, model_sha1)
    _upload_model_pack(pack_model_path, dest_path)
    os.remove(pack_model_path)

    return model_sha1, model_mAP


def _get_class_type_ids(class_types: str) -> Optional[Set[int]]:
    """
    get class type ids of each class names in `class_types`\n
    class id and name mapping is stored in file: type_id_names.csv
    Args:
        class_types: class type names, seperated by ;, for example: person; dog
            empty str means get all types
    Returns:
        None: all types
        empty set: all class type names are not found, or nothing found in type_id_names.csv
        normal set: ids for each type, order of ids is not the same as type names in arg
    """
    if not class_types:
        return None

    cls_mgr = ids.ClassIdManager()
    return set(cls_mgr.id_for_names(class_types.split(";")))


def _find_models(model_root: str) -> Tuple[List[str], float]:
    """
    find models in `model_root`, and returns model names and mAP

    Args:
        model_root (str): model root

    Returns:
        Tuple[List[str], float]: list of model names and map
    """
    model_names = []
    model_mAP = 0

    with open(os.path.join(model_root, "result.yaml"), "r") as f:
        yaml_obj = yaml.safe_load(f.read())
        model_names = yaml_obj["model"]
        model_mAP = float(yaml_obj["map"])

    return ([os.path.join(model_root, name) for name in model_names], model_mAP)


def _pack_models(model_paths: List[str], dest_path: str) -> bool:
    if not model_paths or not dest_path:
        raise ValueError("invalid model_paths or dest_path")

    with tarfile.open(dest_path, "w:gz") as dest_tar_gz:
        for model_path in model_paths:
            logging.info("added {} as {}".format(model_path, os.path.basename(model_path)))
            dest_tar_gz.add(model_path, os.path.basename(model_path))
    return True


def _upload_model_pack(model_pack_path: str, dest_path: str) -> bool:
    # TODO: UPLOAD TO MODEL SERVER WHEN WE HAVE
    if not model_pack_path or not dest_path:
        raise ValueError("invalid model_pack_path or dest_path")

    shutil.copyfile(model_pack_path, dest_path)
    return True


def _update_mir_tasks(mir_root: str, base_branch: str, dst_branch: str, task_id: str, model_sha1: str, mAP: float):
    """
    add a new mir single task into mir_tasks from branch base_task_id, and save it to a new branch: task_id
    """
    task = mirpb.Task()
    task.type = mir_common.TaskTypeTraining
    task.name = "training done"
    task.task_id = task_id
    task.timestamp = int(datetime.datetime.now().timestamp())
    task.model.model_hash = model_sha1
    task.model.mean_average_precision = mAP

    mir_tasks = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
                                                          mir_branch=base_branch,
                                                          ms=mir_common.MirStorage.MIR_TASKS)
    mir_storage_ops.add_mir_task(mir_tasks, task)
    mir_storage_ops.MirStorageOps.save_and_commit(mir_root=mir_root,
                                                  mir_branch=dst_branch,
                                                  his_branch=base_branch,
                                                  mir_datas={mir_common.MirStorage.MIR_TASKS: mir_tasks},
                                                  commit_message=task_id)

    logging.info("task id: {}, model hash: {}, mAP: {}".format(task_id, model_sha1, mAP))


# add this function for mock unit test.
def _run_train_cmd(cmd: str):
    logging.info("training with cmd: {}".format(cmd))
    run_result = subprocess.run(cmd.split(" "))  # run and wait

    # when train process done
    if run_result.returncode != 0:
        logging.error("training error occured: {}".format(run_result.returncode))
        return MirCode.RC_ERROR_UNKNOWN

    return MirCode.RC_OK


def _generate_config(out_config_path: str, task_id: str, gpus: str, class_names: List[str]):
    with open(str(pathlib.Path(__file__).parent.parent.joinpath("conf/training-template.yaml")), "r") as f:
        config = yaml.safe_load(f)

    config["task_id"] = task_id
    config["gpu_train"] = gpus or ""
    config["class_names"] = class_names
    config["class_num"] = len(class_names)
    # config['epochs'] = 20  # ! for test

    with open(out_config_path, "w") as f:
        yaml.dump(config, f)


class CmdTrain(base.BaseCommand):
    def run(self):
        logging.debug("command train: %s", self.args)

        return CmdTrain.run_with_args(type_names=self.args.predicates,
                                      export_root=self.args.work_dir,
                                      model_upload_location=self.args.model_path,
                                      src_revs=self.args.src_revs,
                                      dst_rev=self.args.dst_rev,
                                      mir_root=self.args.mir_root,
                                      gpu=self.args.gpu,
                                      media_location=self.args.media_location,
                                      executor=self.args.executor)

    @staticmethod
    def run_with_args(type_names: str,
                      export_root: str,
                      model_upload_location: str,
                      executor: str,
                      src_revs: str,
                      dst_rev: str,
                      mir_root: str = None,
                      media_location: str = None,
                      gpu: str = None) -> int:
        if not model_upload_location:
            logging.error("invalid model upload location; abort")
            return MirCode.RC_ERROR_INVALID_ARGS
        src_typ_rev_tid = revs_parser.parse_single_arg_rev(src_revs)
        if not src_revs or not src_typ_rev_tid.rev:
            logging.error("invalid --src-revs; abort")
            return MirCode.RC_ERROR_INVALID_ARGS
        dst_typ_rev_tid = revs_parser.parse_single_arg_rev(dst_rev)
        if not dst_rev or not dst_typ_rev_tid.rev or not dst_typ_rev_tid.tid:
            logging.error("invalid --dst-rev; abort")
            return MirCode.RC_ERROR_INVALID_ARGS
        if not export_root:
            logging.error("empty export_root: abort")
            return MirCode.RC_ERROR_INVALID_ARGS

        if not mir_root:
            mir_root = "."

        return_code = checker.check(mir_root)
        if return_code != MirCode.RC_OK:
            return return_code

        task_id = dst_typ_rev_tid.tid
        base_task_id = src_typ_rev_tid.tid
        if not base_task_id:
            mir_tasks = mir_storage_ops.MirStorageOps.load_single(mir_root=mir_root,
                                                                  mir_branch=src_typ_rev_tid.rev,
                                                                  ms=mir_common.MirStorage.MIR_TASKS)
            base_task_id = mir_tasks.head_task_id
        if not base_task_id:
            logging.error("can not find base task id from both --dst-rev or head task id")
            return MirCode.RC_ERROR_INVALID_ARGS

        # get train_ids, val_ids, test_ids
        train_ids = set()  # type: Set[str]
        val_ids = set()  # type: Set[str]
        test_ids = set()  # type: Set[str]
        unused_ids = set()  # type: Set[str]
        mir_datas = mir_storage_ops.MirStorageOps.load(mir_root=mir_root,
                                                       mir_branch=src_typ_rev_tid.rev,
                                                       mir_storages=[mir_common.MirStorage.MIR_METADATAS])
        mir_metadatas = mir_datas[mir_common.MirStorage.MIR_METADATAS]
        for asset_id, asset_attr in mir_metadatas.attributes.items():
            if asset_attr.tvt_type == mir_common.TvtTypeTraining:
                train_ids.add(asset_id)
            elif asset_attr.tvt_type == mir_common.TvtTypeValidation:
                val_ids.add(asset_id)
            elif asset_attr.tvt_type == mir_common.TvtTypeTest:
                test_ids.add(asset_id)
            else:
                unused_ids.add(asset_id)
        if not train_ids:
            logging.error("no training set; abort")
            return MirCode.RC_ERROR_INVALID_ARGS
        if not val_ids:
            logging.error("no validation set; abort")
            return MirCode.RC_ERROR_INVALID_ARGS

        if not unused_ids:
            logging.info(f"training: {len(train_ids)}, validation: {len(val_ids)}, test: {len(test_ids)}")
        else:
            logging.warning(f"training: {len(train_ids)}, validation: {len(val_ids)}, test: {len(test_ids)}, "
                            f"unused: {len(unused_ids)}")

        # export
        logging.info("exporting assets")
        os.makedirs(export_root, exist_ok=True)
        work_dir_in = os.path.join(export_root, "in")
        work_dir_out = os.path.join(export_root, "out")
        os.makedirs(work_dir_in, exist_ok=True)
        os.makedirs(work_dir_out, exist_ok=True)

        type_ids_set = _get_class_type_ids(type_names)
        if not type_ids_set:
            logging.info("type ids empty, please check -p args")
            return MirCode.RC_ERROR_INVALID_ARGS

        # export train set
        work_dir_in_train = os.path.join(work_dir_in, 'train')
        if os.path.isdir(work_dir_in_train):
            shutil.rmtree(work_dir_in_train)
        data_exporter.export(mir_root=mir_root,
                             assets_location=media_location,
                             class_type_ids=type_ids_set,
                             asset_ids=train_ids,
                             asset_dir=work_dir_in_train,
                             annotation_dir=work_dir_in_train,
                             need_suffix=True,
                             base_branch=src_typ_rev_tid.rev,
                             format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK)
        data_exporter.generate_asset_index_file(asset_dir=work_dir_in_train,
                                                rel_start_dir=work_dir_in_train,
                                                index_file_path=os.path.join(work_dir_in_train, 'index.tsv'))

        # export validation set
        work_dir_in_val = os.path.join(work_dir_in, 'val')
        if os.path.isdir(work_dir_in_val):
            shutil.rmtree(work_dir_in_val)
        data_exporter.export(mir_root=mir_root,
                             assets_location=media_location,
                             class_type_ids=type_ids_set,
                             asset_ids=val_ids,
                             asset_dir=work_dir_in_val,
                             annotation_dir=work_dir_in_val,
                             need_suffix=True,
                             base_branch=src_typ_rev_tid.rev,
                             format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK)
        data_exporter.generate_asset_index_file(asset_dir=work_dir_in_val,
                                                rel_start_dir=work_dir_in_val,
                                                index_file_path=os.path.join(work_dir_in_val, 'index.tsv'))

        # export test set (if we have)
        if test_ids:
            work_dir_in_test = os.path.join(work_dir_in, 'test')
            if os.path.isdir(work_dir_in_test):
                shutil.rmtree(work_dir_in_test)
            data_exporter.export(mir_root=mir_root,
                                 assets_location=media_location,
                                 class_type_ids=type_ids_set,
                                 asset_ids=test_ids,
                                 asset_dir=work_dir_in_test,
                                 annotation_dir=work_dir_in_test,
                                 need_suffix=True,
                                 base_branch=src_typ_rev_tid.rev,
                                 format_type=data_exporter.ExportFormat.EXPORT_FORMAT_ARK)
            data_exporter.generate_asset_index_file(asset_dir=work_dir_in_test,
                                                    rel_start_dir=work_dir_in_test,
                                                    index_file_path=os.path.join(work_dir_in_test, 'index.tsv'))

        logging.info("starting train docker container")

        # generate configs
        _generate_config(out_config_path=os.path.join(work_dir_in, "config.yaml"),
                         task_id=task_id,
                         gpus=gpu,
                         class_names=ids.ClassIdManager().all_main_names())

        # start train docker and wait
        path_binds = []
        path_binds.append(f"-v {work_dir_in}:/in")
        path_binds.append(f"-v {work_dir_out}:/out")
        joint_path_binds = " ".join(path_binds)
        cmd = (f"nvidia-docker run -it --rm --shm-size=128G {joint_path_binds} --user {os.getuid()}:{os.getgid()} "
               f"{executor}")  # mir-train-gpu:0.1.1.210824

        ret = _run_train_cmd(cmd)
        if ret != MirCode.RC_OK:
            return ret

        # save model
        logging.info("saving models")
        model_sha1, model_mAP = _process_model_storage(work_dir_out, model_upload_location)
        # update metadatas and task with finish state and model hash
        logging.info("creating task")
        _update_mir_tasks(mir_root=mir_root,
                          base_branch=base_task_id,
                          dst_branch=dst_typ_rev_tid.rev,
                          task_id=task_id,
                          model_sha1=model_sha1,
                          mAP=model_mAP)

        logging.info("training done")

        return MirCode.RC_OK


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    train_arg_parser = subparsers.add_parser("train",
                                             parents=[parent_parser],
                                             description="use this command to train current workspace",
                                             help="train current workspace")
    train_arg_parser.add_argument("-p",
                                  required=True,
                                  dest="predicates",
                                  type=str,
                                  help="predicates to annotations, separated by ;")
    train_arg_parser.add_argument("--model-location",
                                  required=True,
                                  dest="model_path",
                                  type=str,
                                  help="storage place (upload location) to store packed model")
    train_arg_parser.add_argument("--media-location",
                                  required=True,
                                  dest="media_location",
                                  type=str,
                                  help="media storage location for models")
    train_arg_parser.add_argument("--gpu",
                                  dest="gpu",
                                  type=str,
                                  help="gpu ids for training, sep by commas, if not set, uses cpu")
    train_arg_parser.add_argument("-w", required=True, dest="work_dir", type=str, help="work place for training")
    train_arg_parser.add_argument("--executor",
                                  required=True,
                                  dest="executor",
                                  type=str,
                                  help="docker image name for training")
    train_arg_parser.add_argument("--src-revs", dest="src_revs", type=str, help="rev@bid: source rev and base task id")
    train_arg_parser.add_argument("--dst-rev",
                                  dest="dst_rev",
                                  type=str,
                                  required=True,
                                  help="rev@tid: destination branch name and task id")
    train_arg_parser.set_defaults(func=CmdTrain)



import os

from mir.scm.cmd import CmdScm


def Scm(root_dir, scm_executable=None):
    """Returns SCM instance that corresponds to a repo at the specified
        path.
        Args:
            root_dir (str): path to a root directory of the repo.
            scm_excutable(str): one of "dvc" or "git".
        Returns:
            mir.scm.cmd.BaseScm: SCM instance.
        """

    if scm_executable not in ["dvc", "git"]:
        raise ValueError("Args error: expect dvc or git, not %s" % scm_executable)
    if not os.path.exists(root_dir):
        os.makedirs(root_dir)
    if not os.path.isdir(root_dir):
        raise ValueError("Cannot create dir: %s" % root_dir)
    return CmdScm(root_dir, scm_executable)



import logging

from mir import scm
from mir.commands import base
from mir.tools import checker, code


class CmdReset(base.BaseCommand):
    @staticmethod
    def run_with_args(mir_root: str, reset_hard: bool) -> int:
        return_code = checker.check(mir_root,
                                    [checker.Prerequisites.IS_INSIDE_MIR_REPO, checker.Prerequisites.IS_DIRTY])
        if return_code != code.MirCode.RC_OK:
            return return_code

        repo_git = scm.Scm(mir_root, scm_executable="git")
        output_str = repo_git.reset("--hard" if reset_hard else None)
        if output_str:
            logging.info("\n%s" % output_str)

        if reset_hard:
            repo_dvc = scm.Scm(mir_root, scm_executable="dvc")
            output_str = repo_dvc.checkout()
            if output_str:
                logging.info("\n%s" % output_str)

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command reset: %s", self.args)

        return CmdReset.run_with_args(self.args.mir_root, self.args.reset_hard)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    reset_arg_parser = subparsers.add_parser("reset",
                                             parents=[parent_parser],
                                             description="use this command to undo changes to mir repo",
                                             help="undo changes to repo")
    reset_arg_parser.add_argument("--hard",
                                  dest="reset_hard",
                                  action="store_true",
                                  help="hard reset mode: undo add process and also undo the changes")
    reset_arg_parser.set_defaults(func=CmdReset)



import hashlib


def sha1sum_for_file(file_path: str) -> str:
    """
    get sha1sum for file, raises FileNotFoundError if file not found
    """
    h = hashlib.sha1()
    with open(file_path, "rb") as f:
        chunk = 0
        while chunk != b'':
            chunk = f.read(h.block_size)
            h.update(chunk)
    return h.hexdigest()


def md5sum_for_file(file_path: str) -> str:
    """
    get md5sum for file, raises FileNotFoundError if file not found
    """
    h = hashlib.md5()
    with open(file_path, "rb") as f:
        chunk = 0
        while chunk != b'':
            chunk = f.read(h.block_size)
            h.update(chunk)
    return h.hexdigest()



#    Modified from GitPython CMD wrapper:
#    https://github.com/gitpython-developers/GitPython/blob/master/git/cmd.py


class BaseScm():
    """
    Base class providing an interface to retrieve attribute values upon
    first access.
    """

    __slots__ = tuple()

    def __getattr__(self, attr):
        self._set_cache_(attr)
        # will raise in case the cache was not created
        return object.__getattribute__(self, attr)

    def _set_cache_(self, attr):
        """
        This method should be overridden in the derived class.
        """
        pass



import logging

from mir import scm
from mir.commands import base
from mir.tools import checker, code


class CmdBranch(base.BaseCommand):
    @staticmethod
    def run_with_args(mir_root: str, remote: bool, delete: str, force_delete: str) -> int:
        return_code = checker.check(mir_root, [checker.Prerequisites.IS_INSIDE_MIR_REPO])
        if return_code != code.MirCode.RC_OK:
            return return_code

        # can not delete master branch
        if delete and force_delete:
            return code.MirCode.RC_ERROR_INVALID_ARGS
        if delete == "master" or force_delete == "master":
            logging.info("can not delete master branch")
            return code.MirCode.RC_ERROR_INVALID_BRANCH_OR_TAG

        cmd_opts = []
        if remote:
            cmd_opts.append("--remote")
        elif delete:
            cmd_opts.extend(["-d", delete])
        elif force_delete:
            cmd_opts.extend(["-D", force_delete])

        repo_git = scm.Scm(mir_root, scm_executable="git")
        output_str = repo_git.branch(cmd_opts)
        if output_str:
            logging.info("\n%s" % output_str)

        return code.MirCode.RC_OK

    def run(self):
        logging.debug("command branch: %s" % self.args)

        return CmdBranch.run_with_args(mir_root=self.args.mir_root,
                                       remote=False,
                                       delete=self.args.delete,
                                       force_delete=self.args.force_delete)


def bind_to_subparsers(subparsers, parent_parser):  # pragma: no cover
    branch_arg_parser = subparsers.add_parser("branch",
                                              parents=[parent_parser],
                                              description="use this command to show mir repo branches",
                                              help="show mir repo branches")
    delete_group = branch_arg_parser.add_mutually_exclusive_group()
    group = delete_group.add_mutually_exclusive_group()
    group.add_argument("-d", dest="delete", type=str, help="delete branch, if branch not merged, raise errors")
    group.add_argument("-D", dest="force_delete", type=str, help="delete branch, even if branch not merged")
    branch_arg_parser.set_defaults(func=CmdBranch)
